import numpy as np
import pickle
from sklearn.model_selection import train_test_split
from sklearn.metrics import classification_report, confusion_matrix, accuracy_score
from sklearn.svm import SVC
from sklearn.ensemble import RandomForestClassifier
from sklearn.preprocessing import StandardScaler
from lazypredict.Supervised import LazyClassifier
from scipy import signal, stats
import seaborn as sns
import matplotlib.pyplot as plt

from server.sl8 import FileBase
from extra import design_filters


def feature_extraction(data_):
    # data_ = data_ + abs(np.min(data_))
    # ave = np.mean(data_)
    var = np.var(data_)
    std = np.sqrt(var)
    rms = np.sqrt(np.mean(data_ ** 2))
    skewness = stats.skew(data_)
    kurtosis = stats.kurtosis(data_, fisher=False)
    ptop = np.max(data_) - np.min(data_)
    auc = int(np.trapz(data_, dx=1))
    zero_crossings = np.where(np.diff(np.sign(data_)))[0]

    # f, pxx = signal.welch(data_, fs=sr,
    #                       window=signal.windows.tukey(len(data_), sym=False, alpha=.17),
    #                       nperseg=len(data_), noverlap=int(len(data_) * 0.75),
    #                       nfft=2 * sr, return_onesided=True,
    #                       scaling='spectrum', axis=-1, average='mean')
    # psd = np.sum(np.log10(pxx[2:6]))

    # print(ave, var, std, rms, skewness, kurtosis, ptop, auc, len(zero_crossings), psd)
    return [var, std, rms, skewness, kurtosis, ptop, auc, len(zero_crossings)]
    # return [var, std, rms, skewness, kurtosis, ptop, psd]


# filename = './iec_files/1615632656_EyeOpen_1_Other.iec'
# filename = './iec_files/1618321042_EyeOpen_6_sina_izani.iec'
# filename = './iec_files/1618747412_EyeOpen_7_melika_hoseinzadeh.iec'
# filename = './iec_files/1619683416_EyeOpen_9_khosravi.iec'
# filename = './iec_files/1620309313_EyeOpen_12_mr_mohsen_shahbaz_beigi.iec'
# filename = './iec_files/1620576311_EyeOpen_14_mahan_endallah.iec'
# filename = './iec_files/1621949668_EyeOpen_16_diyana_ebrahimi_0.iec'
# filename = './iec_files/1623149547_EyeOpen_1_Other_3.iec'
# filename = './iec_files/1623154705_EyeOpen_1_Other_3.iec'
# filename = './iec_files/1623597049_EyeOpen_18_anita_mirzayi_0.iec'
# filename = './iec_files/1624281526_EyeOpen_1_Other_0.iec'
# filename = './iec_files/1624281967_EyeOpen_1_Other_0.iec'
# file_list = [filename]

file_list = ['./iec_files/1615632656_EyeOpen_1_Other.iec',
             './iec_files/1618321042_EyeOpen_6_sina_izani.iec',
             './iec_files/1618747412_EyeOpen_7_melika_hoseinzadeh.iec',
             './iec_files/1619683416_EyeOpen_9_khosravi.iec',
             './iec_files/1620309313_EyeOpen_12_mr_mohsen_shahbaz_beigi.iec',
             './iec_files/1620576311_EyeOpen_14_mahan_endallah.iec',
             './iec_files/1621949668_EyeOpen_16_diyana_ebrahimi_0.iec',
             './iec_files/1623149547_EyeOpen_1_Other_3.iec',
             './iec_files/1623154705_EyeOpen_1_Other_3.iec',
             './iec_files/1623597049_EyeOpen_18_anita_mirzayi_0.iec',
             './iec_files/1624281526_EyeOpen_1_Other_0.iec',
             './iec_files/1624281967_EyeOpen_1_Other_0.iec'
             ]

feature_list = []
for filename in file_list:
    labeled_filename = filename.replace('iec_files', 'label_results').replace('iec', 'txt')

    iec = FileBase()
    iec.load(filename)
    data = iec.get_brain_raw_data()
    sr = iec.get_brain_sample_rate()
    gain = iec.get_gain()

    data = data / gain
    data = signal.detrend(data, axis=1)
    filters = design_filters(sr)
    data = signal.filtfilt(filters['notch1'][0], filters['notch1'][1], data)
    data = signal.filtfilt(filters['notch2'][0], filters['notch2'][1], data)
    data = signal.filtfilt(filters['bandpass'][0], filters['bandpass'][1], data)

    # with open(labeled_filename) as f:
    #     content = f.readlines()
    # # you may also want to remove whitespace characters like `\n` at the end of each line
    # content = [x.strip()[1:-1] for x in content]
    #
    # arr = np.zeros((len(content), 4), dtype=int)
    # for i in range(len(content)):
    #     temp = content[i].split(',')
    #     arr[i, 0] = int(temp[0])
    #     arr[i, 1] = int(temp[1])
    #     arr[i, 2] = int(temp[2])
    #     arr[i, 3] = int(temp[3])
    # del content
    arr = np.loadtxt(labeled_filename, dtype=int)

    features = []
    for i in range(arr.shape[0]):
        # print(arr[i, 0], arr[i, 1], arr[i, 2])
        my_data = data[arr[i, 0], arr[i, 1]: arr[i, 2]]
        # my_data = my_data / gain
        # my_data -= np.mean(my_data)
        # my_data = signal.detrend(my_data)

        # my_data = signal.filtfilt(filters['notch1'][0], filters['notch1'][1], my_data)
        # my_data = signal.filtfilt(filters['notch2'][0], filters['notch2'][1], my_data)
        # my_data = signal.filtfilt(filters['bandpass'][0], filters['bandpass'][1], my_data)

        # my_data = (my_data - np.min(my_data)) / (np.max(my_data) - np.min(my_data))
        # my_data = my_data / np.max(my_data)
        # my_data = (my_data - np.min(my_data)) / np.ptp(my_data)

        feats = feature_extraction(my_data)
        feats.append(arr[i, 3])
        features.append(feats)

    features = np.array(features)

    features = features[features[:, -1].argsort()]
    a = features[features[:, -1] == 0]
    b = features[features[:, -1] == 1]
    c = a[np.random.randint(a.shape[0], size=b.shape[0]), :]
    features = np.vstack((c, b))
    feature_list.append(features)

X_all = feature_list[0]
for i in range(0, len(feature_list) - 1):
    X_all = np.concatenate((X_all, feature_list[i + 1]), axis=0)

X = X_all[:, :-1]
y = X_all[:, -1]

a = y[y == 0]
b = y[y == 1]
print(len(a), len(b))

scaler = StandardScaler()
scaled_data = scaler.fit_transform(X)

X_train, X_test, y_train, y_test = train_test_split(scaled_data, y, test_size=0.30)

svclassifier = SVC(kernel='rbf', class_weight={1: len(a) / len(b)}, probability=True)
svclassifier.fit(X_train, y_train)

y_pred = svclassifier.predict(X_test)

cm = confusion_matrix(y_test, y_pred)
acc = accuracy_score(y_test, y_pred)
print('Accuracy:', acc)
print(cm)
print(classification_report(y_test, y_pred))
plt.figure()
ax = plt.subplot()
plt.suptitle('Accuracy: ' + str(acc))
sns.heatmap(cm, annot=True, ax=ax, fmt='g', cmap='Greens')
plt.show()

# # save the classifier
# model = {'model': svclassifier, 'scaler': scaler}
# with open('svclassifier_new3.pkl', 'wb') as fid:
#     pickle.dump(model, fid)

# rfc = RandomForestClassifier(max_depth=10, random_state=0)
# rfc.fit(X_train, y_train)
#
# y_pred = rfc.predict(X_test)
#
# cm = confusion_matrix(y_test, y_pred)
# acc = accuracy_score(y_test, y_pred)
# print('Accuracy:', acc)
# print(cm)
# print(classification_report(y_test, y_pred))
# plt.figure()
# ax = plt.subplot()
# plt.suptitle('Accuracy: ' + str(acc))
# sns.heatmap(cm, annot=True, ax=ax, fmt='g', cmap='Greens')
# plt.show()

# clf = LazyClassifier(verbose=0, ignore_warnings=True, custom_metric=None)
# models, predictions = clf.fit(X_train, X_test, y_train, y_test)
# print(models)
