import numpy as np

from blink_remove import blinkremove, eegplot
from scipy import signal

from server.sl8 import FileBase
from extra import design_filters, iec_maker

file_list = []

for filename in file_list:
    try:
        iec = FileBase()
        iec.load(filename)
        data = iec.get_brain_raw_data()
        sr = iec.get_brain_sample_rate()
        chnum = iec.get_brain_channel_count()
        gain = iec.get_gain()
        lead_off = iec.get_leadoff()

        filters = design_filters(sr)
        data = signal.filtfilt(filters['notch1'][0], filters['notch1'][1], data)
        data = signal.filtfilt(filters['notch2'][0], filters['notch2'][1], data)
        data = signal.filtfilt(filters['bandpass'][0], filters['bandpass'][1], data)

        data = data[:, 1024:-1024]

        eegplot(data, sr, 'EEG before ICA')
        clean_data = blinkremove(data, sr, chnum, debug_mode=False)
        eegplot(clean_data, sr, 'EEG after ICA')

        iec_maker(clean_data, filename.split('/')[-1].split('.')[0], sr)
    except Exception as e:
        print(e)
