import numpy as np
import pickle
import matplotlib.pyplot as plt
from scipy import signal, stats
from keras.models import load_model

from server.sl8 import FileBase
from extra import design_filters
from blink_remove import eegplot


def feature_extraction(data_):
    var = np.var(data_)
    std = np.sqrt(var)
    rms = np.sqrt(np.mean(data_ ** 2))
    skewness = stats.skew(data_)
    kurtosis = stats.kurtosis(data_, fisher=False)
    ptop = np.max(data_) - np.min(data_)
    auc = int(np.trapz(data_, dx=1))
    zero_crossings = np.where(np.diff(np.sign(data_)))[0]

    return [var, std, rms, skewness, kurtosis, ptop, auc, len(zero_crossings)]


# load classifier
with open('svclassifier_new3.pkl', 'rb') as fid:
    my_model = pickle.load(fid)
model = my_model['model']
scaler = my_model['scaler']
conv_model = load_model('conv_model.h5')

file_list = ['./iec_files/1615632656_EyeOpen_1_Other.iec',
             './iec_files/1618321042_EyeOpen_6_sina_izani.iec',
             './iec_files/1618747412_EyeOpen_7_melika_hoseinzadeh.iec',
             './iec_files/1619683416_EyeOpen_9_khosravi.iec',
             './iec_files/1620309313_EyeOpen_12_mr_mohsen_shahbaz_beigi.iec',
             './iec_files/1620576311_EyeOpen_14_mahan_endallah.iec',
             './iec_files/1621949668_EyeOpen_16_diyana_ebrahimi_0.iec',
             './iec_files/1623149547_EyeOpen_1_Other_3.iec',
             './iec_files/1623154705_EyeOpen_1_Other_3.iec',
             './iec_files/1623597049_EyeOpen_18_anita_mirzayi_0.iec',
             './iec_files/1624281526_EyeOpen_1_Other_0.iec',
             './iec_files/1624281967_EyeOpen_1_Other_0.iec'
             ]
file_list = [
    './test_iec/1624793690_EyeOpen_20_afshari_0.iec',
    # './test_iec/1625637362_EyeOpen_5_maryam_miralmasi_3.iec'
    # './test_iec/1625758306_EyeOpen_7_asghar_Azimi_3.iec'
]

# for file in file_list:
#     iec = FileBase()
#     iec.load(file)
#     data = iec.get_brain_raw_data()
#     sr = iec.get_brain_sample_rate()
#     sc = iec.get_brain_sample_count()
#     gain = iec.get_gain()
#
#     # data = data / gain
#
#     eegplot(data, sr, 'EEG')
#
#     filters = design_filters(sr)
#
#     for i in range(2):
#         for j in range(0, len(data[i]) - 512, 512):
#             lst = []
#             my_data = data[i, j: j + 512]
#
#             my_data = signal.filtfilt(filters['notch1'][0], filters['notch1'][1], my_data)
#             my_data = signal.filtfilt(filters['notch2'][0], filters['notch2'][1], my_data)
#             my_data = signal.filtfilt(filters['bandpass'][0], filters['bandpass'][1], my_data)
#
#             my_data = my_data / gain
#             # my_data -= np.mean(my_data)
#             my_data = signal.detrend(my_data)
#
#             for k in range(0, len(my_data) - 100, 40):
#                 new_data = my_data[k: k + 100]
#
#                 features = np.array(feature_extraction(new_data))
#                 temp = scaler.transform(features.reshape(1, features.shape[0]))
#                 state = model.predict(temp)
#
#                 if state:
#                     print('Blink', i, j, j + 512, k, k + 100, model.predict_proba(temp)[0, 1])
#
#                     a = np.min(new_data)
#                     b = np.max(new_data)
#                     samples = (new_data - a) / (b - a)
#                     samples = samples.reshape((1, new_data.shape[0], 1))
#                     predicted = conv_model.predict(samples).squeeze()
#                     predicted = predicted * (b - a) + a
#                     # my_data[k: k+100] = predicted
#                     lst.append([i, j, j + 512, k, k + 100, predicted])
#                     # plt.plot(my_data)
#                     # plt.axvline(x=k, color='r', linestyle='-')
#                     # plt.axvline(x=k + 100, color='r', linestyle='-')
#                     # plt.show()
#                 else:
#                     print('Clean')
#             if len(lst) > 0:
#                 start = lst[0][3]
#                 end = lst[0][4]
#
#                 fig, axes = plt.subplots(2, 2)
#                 axes[0, 0].plot(my_data)
#                 axes[0, 0].set_title('before')
#                 f, pxx = signal.welch(my_data, fs=sr,
#                                       window=signal.windows.tukey(len(my_data), sym=False, alpha=.17),
#                                       nperseg=len(my_data), noverlap=int(len(my_data) * 0.75),
#                                       nfft=2 * sr, return_onesided=True,
#                                       scaling='spectrum', axis=-1, average='mean')
#                 a = pxx
#                 axes[1, 0].plot(f, pxx)
#                 axes[1, 0].set_xlim((0, 30))
#                 axes[1, 0].set_title('psd')
#                 # main_mean = np.mean(my_data)
#                 # win_mean = np.mean(lst[0][5])
#                 # lst[0][5] = lst[0][5] + (main_mean - win_mean)
#                 my_data[start: end] = lst[0][5]
#                 if len(lst) > 1:
#                     for x in range(1, len(lst)):
#                         start_new = lst[x][3]
#                         end_new = lst[x][4]
#                         # win_mean = np.mean(lst[x][5])
#                         # lst[x][5] = lst[x][5] + (main_mean - win_mean)
#                         if end_new - end > 100:
#                             my_data[start_new: end_new] = lst[x][5]
#                             end = end_new
#                         else:
#                             my_data[end: end_new] = lst[x][5][-(end_new - end):]
#                             end = end_new
#                 axes[0, 1].plot(my_data)
#                 axes[0, 1].set_title('after')
#                 f, pxx = signal.welch(my_data, fs=sr,
#                                       window=signal.windows.tukey(len(my_data), sym=False, alpha=.17),
#                                       nperseg=len(my_data), noverlap=int(len(my_data) * 0.75),
#                                       nfft=2 * sr, return_onesided=True,
#                                       scaling='spectrum', axis=-1, average='mean')
#                 b = pxx
#                 axes[1, 1].plot(f, pxx)
#                 axes[1, 1].set_xlim((0, 30))
#                 axes[1, 1].set_title('psd')
#                 plt.show()
#
#                 f, Cxy = signal.coherence(a, b, sr,
#                                           nperseg=len(my_data) // 8, noverlap=int(len(my_data) // 8 * 0.5),
#                                           nfft=2 * sr, axis=-1)
#                 coh = np.sum(Cxy[2:60], axis=0)
#                 plt.title(str(coh)+' / 60')
#                 plt.plot(f, Cxy)
#                 plt.xlim((0, 30))
#                 plt.show()
#                 plt.figure()
#                 plt.plot(f, a)
#                 plt.plot(f, b)
#                 plt.xlim((0, 30))
#                 plt.show()
#     # eegplot(my_data, sr, 'EEG')


file = file_list[0]
iec = FileBase()
iec.load(file)
iec_data = iec.get_brain_raw_data()
sr = iec.get_brain_sample_rate()
sc = iec.get_brain_sample_count()
gain = iec.get_gain()

# eegplot(iec_data, sr, 'EEG')

filters = design_filters(sr)
iec_data = np.array(iec_data)

for k in range(0, iec_data.shape[1] - 512, 512):
    org_data = iec_data[:, k:k+512]
    self_data = org_data / gain
    self_data = signal.detrend(self_data, axis=1)
    self_data = signal.filtfilt(filters['notch1'][0], filters['notch1'][1], self_data)
    self_data = signal.filtfilt(filters['notch2'][0], filters['notch2'][1], self_data)
    self_data = signal.filtfilt(filters['bandpass'][0], filters['bandpass'][1], self_data)
    for i in range(2):
        lst = []
        data = self_data[i, :]
        for j in range(0, data.shape[0] - 100, 40):
            new_data = data[j: j + 100]
            features = np.array(feature_extraction(new_data))
            temp = scaler.transform(features.reshape(1, features.shape[0]))
            state = model.predict(temp)
            if state:
                print('Blink', i, j, j + 100, model.predict_proba(temp)[0, 1])
                a = np.min(new_data)
                b = np.max(new_data)
                samples = (new_data - a) / (b - a)
                samples = samples.reshape((1, new_data.shape[0], 1))
                predicted = conv_model.predict(samples).squeeze()
                predicted = predicted * (b - a) + a
                lst.append([i, j, j + 100, predicted])
            else:
                print('clean')
        if len(lst) > 0:
            fig, axes = plt.subplots(2, 2)
            axes[0, 0].plot(data)
            axes[0, 0].set_title('before')
            f, pxx = signal.welch(data, fs=sr,
                                  window=signal.windows.tukey(len(data), sym=False, alpha=.17),
                                  nperseg=len(data), noverlap=int(len(data) * 0.75),
                                  nfft=2 * sr, return_onesided=True,
                                  scaling='spectrum', axis=-1, average='mean')
            axes[1, 0].plot(f, pxx)
            axes[1, 0].set_xlim((0, 30))
            axes[1, 0].set_title('psd')
            # main_mean = np.mean(org_data)
            # win_mean = np.mean(lst[0][3])
            # lst[0][3] = lst[0][3] + (main_mean - win_mean)
            start = lst[0][1]
            end = lst[0][2]
            data[start: end] = lst[0][3]
            if len(lst) > 1:
                for x in range(1, len(lst)):
                    start_new = lst[x][1]
                    end_new = lst[x][2]
                    # win_mean = np.mean(lst[x][5])
                    # lst[x][5] = lst[x][5] + (main_mean - win_mean)
                    if end_new - end > 100:
                        data[start_new: end_new] = lst[x][3]
                        end = end_new
                    else:
                        data[end: end_new] = lst[x][3][-(end_new - end):]
                        end = end_new
            axes[0, 1].plot(data)
            axes[0, 1].set_title('after')
            f, pxx = signal.welch(data, fs=sr,
                                  window=signal.windows.tukey(len(data), sym=False, alpha=.17),
                                  nperseg=len(data), noverlap=int(len(data) * 0.75),
                                  nfft=2 * sr, return_onesided=True,
                                  scaling='spectrum', axis=-1, average='mean')
            axes[1, 1].plot(f, pxx)
            axes[1, 1].set_xlim((0, 30))
            axes[1, 1].set_title('psd')
            plt.show()

            self_data[i, :] = data
